import requests
import json


class UserComment:
    def __init__(self, json_data, _client, _headers):
        self.author = json_data["Username"]
        self._client = _client
        self._headers = _headers
        self.content = json_data["Content"]
        self.post_time = json_data["Time"]
        self.is_reply = json_data["IsReply"]
        self.id = json_data["CommentID"]
        self.replies = [UserComment(i, self._client, self._headers) for i in json_data["Replies"]]

    def get_user(self):
        return self._client.get_user(self.author)
    def reply(self, content, commentee_id=""):
        data = {
            "commentee_id": commentee_id,
            "content": content,
            "parent_id": self.id,
        }
        requests.post(
            "https://scratch.mit.edu/site-api/comments/user/" + self.author + "/add/",
            headers=self._headers,
            data=json.dumps(data),
        )


class ProjectComment:
    def __init__(self, project, data, client):
        self.id = data["id"]
        self.parent_id = data["parent_id"]
        self.commentee_id = data["commentee_id"]
        self.content = data["content"]
        self.reply_count = data["reply_count"]

        self.author = data["author"]["username"]
        self.author_id = data["author"]["id"]

        self.created_timestamp = data["datetime_created"]
        self.last_modified_timestamp = data["datetime_modified"]

        self.visible = data["visibility"] == "visible"

        self.project = project
        self._client = client

    def delete(self):
        requests.delete(
            "https://api.scratch.mit.edu/proxy/comments/project/"
            + str(self.project.id)
            + "/comment/"
            + str(self.id),
            headers=self.project._headers,
        )

    def report(self):
        requests.post(
            "https://api.scratch.mit.edu/proxy/comments/project/"
            + str(self.project.id)
            + "/comment/"
            + str(self.id)
            + "/report",
            headers=self.project._headers,
        )

    def reply(self, content):
        self.project.post_comment(content, self.id, self.author_id)

    def get_user(self):
        return self._client.get_user(self.author)



import requests
import json
from .UserProfiles import YourUserProfile, AnotherUserProfile
from .Project import YourProject, AnotherProject
from bs4 import BeautifulSoup
from .Comments import UserComment
import re


class YourUser:
    def __init__(self, data, client):
        self.id = data["id"]
        self.username = data["username"]
        self.joined_timestamp = data["history"]["joined"]
        self.scratchteam = data["scratchteam"]
        self.scrather = requests.get(f"https://isscratcher.9pfs.repl.co/api/" + self.username).json()["isScratcher"]
        self.profile = YourUserProfile(data["profile"], self)
        self._client = client
        self._headers = {
            "x-csrftoken": self._client.csrf_token,
            "X-Token": self._client.token,
            "x-requested-with": "XMLHttpRequest",
            "Cookie": "scratchcsrftoken="
                      + self._client.csrf_token
                      + ";scratchlanguage=en;scratchsessionsid="
                      + self._client.session_id
                      + ";",
            "referer": "https://scratch.mit.edu/users/" + self.username + "/",
        }

    def get_projects(self, all=False, limit=20, offset=0):
        if all:
            projects = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/projects/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                projects += res
                if len(res) != 40:
                    break
                offset += 40
            for x, i in enumerate(projects):
                projects[x].update({
                    "author": self.username
                })
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]
        else:

            projects = requests.get(
                "https://api.scratch.mit.edu/users/"
                + self.username
                + "/projects/"
                + "?limit="
                + str(limit)
                + "&offset="
                + str(offset)
            ).json(),
            for x, i in enumerate(projects):
                projects[x].update({
                    "author": self.username
                })
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

    def get_curating(self, all=False, limit=20, offset=0):
        if all:
            studios = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/studios/curate"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                studios += res
                if len(res) != 40:
                    break
                offset += 40
            return list(map(self._client.studio, studios))
        else:
            return list(
                map(
                    self._client.studio,
                    requests.get(
                        "https://api.scratch.mit.edu/users/"
                        + self.username
                        + "/studios/curate/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json(),
                )
            )

    def get_favorites(self, all=False, limit=20, offset=0):
        if all:
            projects = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/favorites/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                projects += res
                if len(res) != 40:
                    break
                offset += 40
            for x, i in enumerate(projects):
                projects[x]["author"] = self.username
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

        else:
            projects = list(
                {
                    **requests.get(
                        "https://api.scratch.mit.edu/users/"
                        + self.username
                        + "/favorites/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json(),
                    **{
                        "author": self.username
                    }}
            )
            [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

    def get_followers(self, all=False, limit=20, offset=0):
        if all:
            users = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/followers/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                users += res
                if len(res) != 40:
                    break
                offset += 40
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]
        else:
            users = list(
                requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/followers/"
                    + "?limit="
                    + str(limit)
                    + "&offset="
                    + str(offset)
                ).json(),
            )
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]

    def get_following(self, all=False, limit=20, offset=0):
        if all:
            users = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/following/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                users += res
                if len(res) != 40:
                    break
                offset += 40
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]
        else:
            users = list(
                requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/following/"
                    + "?limit="
                    + str(limit)
                    + "&offset="
                    + str(offset)
                ).json(),
            )
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]

    def get_message_count(self):
        return requests.get(
            "https://api.scratch.mit.edu/users/" + self.username + "/messages/count/"
        ).json()["count"]

    def get_comments(self, page=1):
        comments = []
        soup = BeautifulSoup(
            requests.get(f"https://scratch.mit.edu/site-api/comments/user/{self.username}/?page={page}").content,
            "html.parser")
        result = soup.find_all("li", class_="top-level-reply")

        def get_replies(count):
            '''
            Retrieve replies to comment thread.
            '''
            # Extract reply list
            replies = result[count].find("ul", class_="replies")
            if replies.text == "":
                # Detect empty reply chain
                return None
            else:
                # Get DOM node containing user data for comment
                user = replies.find_all("div", class_="info")
                # print(user)
                # Initialize array with name "all_replies"
                all_replies = []
                # Iterate through reply list and extract username
                for i in range(0, len(user)):
                    # Get username section. Probably does it like this to save memory.
                    username = user[i].find("div", class_="name")
                    # Redefine username as the actual username element
                    username = username.find("a").text
                    # Get post content
                    content = user[i].find("div", class_="content").text
                    # Trim username newlines
                    username = username.strip().replace("\n", "")
                    # Trim post content newlines
                    content = content.strip().replace("\n", "")
                    # Get comment IDs
                    search = re.search("data-comment-id=", str(result[i]))
                    # Get post position in reply list
                    index = search.span()[1]
                    data = str(result[i])[index + 1:]
                    i = 0
                    id = ""
                    while data[i] != '"':
                        id += data[i]
                        i += 1
                    id = int(id)
                    # Get post numbers (I think)
                    search = re.search("title=", str(result[i]))
                    index = search.span()[1]
                    data = str(result[i])[index + 1:]
                    i = 0
                    comment_time = ""
                    while data[i] != '"':
                        comment_time += data[i]
                        i += 1
                    reply = {"id": id, "username": username, "comment": content.replace("                   ", ""),
                             "timestamp": comment_time}
                    all_replies.append(reply)
                return all_replies

        for i in range(0, len(result)):
            user = result[i].find("div", class_="comment")
            replies = get_replies(i)
            user = user.find("div", class_="info")
            user = user.find("div", class_="name")
            user = user.find("a")
            user = user.text

            content = result[i].find("div", class_="comment")
            content = content.find("div", class_="info")
            content = content.find("div", class_="content")
            content = content.text.strip()

            search = re.search("data-comment-id=", str(result[i]))
            index = search.span()[1]
            data = str(result[i])[index + 1:]
            i = 0
            id = ""
            while data[i] != '"':
                id += data[i]
                i += 1
            id = int(id)

            search = re.search("title=", str(result[i]))
            index = search.span()[1]
            data = str(result[i])[index + 1:]
            i = 0
            comment_time = ""
            while data[i] != '"':
                comment_time += data[i]
                i += 1
            if len(replies) == 0:
                parent = True
            else:
                parent = False
            comment = {
                "Username": user,
                "Content": content,
                "Time": comment_time,
                "IsReply": parent,
                "Replies": replies,
                "CommentID": id
            }
            comments.append(comment)
        # Return a list of comments
        return map(lambda data: UserComment(data, self._client, self._headers), comments)

    def post_comment(self, content, parent_id="", commentee_id=""):
        data = {
            "commentee_id": commentee_id,
            "content": content,
            "parent_id": parent_id,
        }
        requests.post(
            "https://scratch.mit.edu/site-api/comments/user/" + self.username + "/add/",
            headers=self._headers,
            data=json.dumps(data),
        )

    def report(self, field):

        data = {"selected_field": field}
        requests.post(
            "https://scratch.mit.edu/site-api/users/all/" + self.username + "/report/",
            headers=self._headers,
            data=json.dumps(data),
        )

    def toggle_commenting(self):

        requests.post(
            "https://scratch.mit.edu/site-api/comments/user/"
            + self.username
            + "/toggle-comments/",
            headers=self._headers,
        )

    def follow(self):
        return requests.put(
            "https://scratch.mit.edu/site-api/users/followers/"
            + self.username
            + "/add/?usernames="
            + self._client.username,
            headers=self._headers,
        ).json()

    def unfollow(self):
        return requests.put(
            "https://scratch.mit.edu/site-api/users/followers/"
            + self.username
            + "/remove/?usernames="
            + self._client.username,
            headers=self._headers,
        ).json()


class AnotherUser:
    def __init__(self, data, client):
        self.id = data["id"]
        self.username = data["username"]
        self.joined_timestamp = data["history"]["joined"]
        self.scratchteam = data["scratchteam"]
        self.deleted = False
        try:
            self.scrather = requests.get(f"https://isscratcher.9pfs.repl.co/api/" + self.username).json()["isScratcher"]
        except:
            self.deleted = True
        self.profile = AnotherUserProfile(data["profile"], self)
        self._client = client
        self._headers = {
            "x-csrftoken": self._client.csrf_token,
            "X-Token": self._client.token,
            "x-requested-with": "XMLHttpRequest",
            "Cookie": "scratchcsrftoken="
                      + self._client.csrf_token
                      + ";scratchlanguage=en;scratchsessionsid="
                      + self._client.session_id
                      + ";",
            "referer": "https://scratch.mit.edu/users/" + self.username + "/",
        }

    def get_projects(self, all=False, limit=20, offset=0):
        if all:
            projects = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/projects/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                projects += res
                if len(res) != 40:
                    break
                offset += 40
            for x, i in enumerate(projects):
                projects[x].update({
                    "author": self.username
                })
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]
        else:

            projects = requests.get(
                "https://api.scratch.mit.edu/users/"
                + self.username
                + "/projects/"
                + "?limit="
                + str(limit)
                + "&offset="
                + str(offset)
            ).json(),
            for x, i in enumerate(projects):
                projects[x].update({
                    "author": self.username
                })
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

    def get_curating(self, all=False, limit=20, offset=0):
        if all:
            studios = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/studios/curate"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                studios += res
                if len(res) != 40:
                    break
                offset += 40
            return list(map(self._client.studio, studios))
        else:
            return list(
                map(
                    self._client.studio,
                    requests.get(
                        "https://api.scratch.mit.edu/users/"
                        + self.username
                        + "/studios/curate/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json(),
                )
            )

    def get_favorites(self, all=False, limit=20, offset=0):
        if all:
            projects = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/favorites/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                projects += res
                if len(res) != 40:
                    break
                offset += 40
            for x, i in enumerate(projects):
                projects[x]["author"] = self.username
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

        else:
            projects = list(
                {
                    **requests.get(
                        "https://api.scratch.mit.edu/users/"
                        + self.username
                        + "/favorites/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json(),
                    **{
                        "author": self.username
                    }}
            )
            [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

    def get_followers(self, all=False, limit=20, offset=0):
        if all:
            users = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/followers/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                users += res
                if len(res) != 40:
                    break
                offset += 40
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]
        else:
            users = list(
                requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/followers/"
                    + "?limit="
                    + str(limit)
                    + "&offset="
                    + str(offset)
                ).json(),
            )
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]

    def get_following(self, all=False, limit=20, offset=0):
        if all:
            users = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/following/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                users += res
                if len(res) != 40:
                    break
                offset += 40
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]
        else:
            users = list(
                requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.username
                    + "/following/"
                    + "?limit="
                    + str(limit)
                    + "&offset="
                    + str(offset)
                ).json(),
            )
            return [
                YourUser(i, self._client) if i["username"] == self._client.username else AnotherUser(i,
                                                                                                     self._client)
                for i in users]

    def get_message_count(self):
        return requests.get(
            "https://api.scratch.mit.edu/users/" + self.username + "/messages/count/"
        ).json()["count"]

    def get_comments(self, page=1):
        comments = []
        soup = BeautifulSoup(
            requests.get(f"https://scratch.mit.edu/site-api/comments/user/{self.username}/?page={page}").content,
            "html.parser")
        result = soup.find_all("li", class_="top-level-reply")

        def get_replies(count):
            '''
            Retrieve replies to comment thread.
            '''
            # Extract reply list
            replies = result[count].find("ul", class_="replies")
            if replies.text == "":
                # Detect empty reply chain
                return None
            else:
                # Get DOM node containing user data for comment
                user = replies.find_all("div", class_="info")
                # print(user)
                # Initialize array with name "all_replies"
                all_replies = []
                # Iterate through reply list and extract username
                for i in range(0, len(user)):
                    # Get username section. Probably does it like this to save memory.
                    username = user[i].find("div", class_="name")
                    # Redefine username as the actual username element
                    username = username.find("a").text
                    # Get post content
                    content = user[i].find("div", class_="content").text
                    # Trim username newlines
                    username = username.strip().replace("\n", "")
                    # Trim post content newlines
                    content = content.strip().replace("\n", "")
                    # Get comment IDs
                    search = re.search("data-comment-id=", str(result[i]))
                    # Get post position in reply list
                    index = search.span()[1]
                    data = str(result[i])[index + 1:]
                    i = 0
                    id = ""
                    while data[i] != '"':
                        id += data[i]
                        i += 1
                    id = int(id)
                    # Get post numbers (I think)
                    search = re.search("title=", str(result[i]))
                    index = search.span()[1]
                    data = str(result[i])[index + 1:]
                    i = 0
                    comment_time = ""
                    while data[i] != '"':
                        comment_time += data[i]
                        i += 1
                    reply = {"id": id, "username": username, "comment": content.replace("                   ", ""),
                             "timestamp": comment_time}
                    all_replies.append(reply)
                return all_replies

        for i in range(0, len(result)):
            user = result[i].find("div", class_="comment")
            replies = get_replies(i)
            user = user.find("div", class_="info")
            user = user.find("div", class_="name")
            user = user.find("a")
            user = user.text

            content = result[i].find("div", class_="comment")
            content = content.find("div", class_="info")
            content = content.find("div", class_="content")
            content = content.text.strip()

            search = re.search("data-comment-id=", str(result[i]))
            index = search.span()[1]
            data = str(result[i])[index + 1:]
            i = 0
            id = ""
            while data[i] != '"':
                id += data[i]
                i += 1
            id = int(id)

            search = re.search("title=", str(result[i]))
            index = search.span()[1]
            data = str(result[i])[index + 1:]
            i = 0
            comment_time = ""
            while data[i] != '"':
                comment_time += data[i]
                i += 1
            if len(replies) == 0:
                parent = True
            else:
                parent = False
            comment = {
                "Username": user,
                "Content": content,
                "Time": comment_time,
                "IsReply": parent,
                "Replies": replies,
                "CommentID": id
            }
            comments.append(comment)
        # Return a list of comments
        return map(lambda data: UserComment(data, self._client, self._headers), comments)

    def post_comment(self, content, parent_id="", commentee_id=""):
        data = {
            "commentee_id": commentee_id,
            "content": content,
            "parent_id": parent_id,
        }
        requests.post(
            "https://scratch.mit.edu/site-api/comments/user/" + self.username + "/add/",
            headers=self._headers,
            data=json.dumps(data),
        )

    def report(self, field):

        data = {"selected_field": field}
        requests.post(
            "https://scratch.mit.edu/site-api/users/all/" + self.username + "/report/",
            headers=self._headers,
            data=json.dumps(data),
        )

    def follow(self):
        return requests.put(
            "https://scratch.mit.edu/site-api/users/followers/"
            + self.username
            + "/add/?usernames="
            + self._client.username,
            headers=self._headers,
        ).json()

    def unfollow(self):
        return requests.put(
            "https://scratch.mit.edu/site-api/users/followers/"
            + self.username
            + "/remove/?usernames="
            + self._client.username,
            headers=self._headers,
        ).json()



import websocket
import json
import time



import threading
from pymitter import EventEmitter
import string


class Encoder:
    def __init__(self, codec: list = list(string.ascii_uppercase + string.ascii_lowercase + string.digits + string.punctuation + ' ')):
        self.codec = codec



class CloudVariable:
    def __init__(self, name: str, value: str):
        self.name = name
        self.value = value

    def __eq__(self, other):
        return self.value == other.value

    def __ne__(self, other):
        return self.value != other.value


class CloudScCodeVariable:
    def __init__(self, name: str, value: str, Encoder):
        self.name = name
        self.value = value
        self.Encoder = Encoder

    def __eq__(self, other):
        return self.value == other.value

    def __ne__(self, other):
        return self.value != other.value

    def __add__(self, other):
        return CloudScCodeVariable(name=self.name,
                                   value=self.value.replace(self.value[0]) + self.value.replace(self.value)[0])


class CloudConnection(EventEmitter):
    def __init__(self, project_id: int, client, website="scratch.mit.edu", ScCode=CloudVariable):
        EventEmitter.__init__(self)
        self.ScCode = ScCode
        self._client = client
        self._website = website
        self.connect(project_id)

    def _send_packet(self, packet):
        self._ws.send(json.dumps(packet) + "\n")

    def connect(self, project_id):
        if project_id:
            self.project_id = project_id
        self._ws = websocket.WebSocket()
        self._cloudvariables = []
        self._timer = time.time()
        self._ws.connect(
            f"wss://clouddata.{self._website}",
            cookie="scratchsessionsid=" + self._client.session_id + ";",
            origin=self._website,
            enable_multithread=True,
        )  # connect the websocket
        self._send_packet(
            {
                "method": "handshake",
                "user": self._client.username,
                "project_id": str(self.project_id),
            }
        )
        self.emit("handshake")
        response = self._ws.recv().split("\n")
        for variable in response:
            try:
                variable = json.loads(str(variable))
            except:
                pass
            else:
                self._cloudvariables.append(
                    self.ScCode(variable["name"], variable["value"])
                )
        self._start_cloud_var_loop()

    def set_cloud_variable(self, variable, value):
        if time.time() - self._timer > 0.1:
            if not str(value).isdigit():
                raise ValueError(
                    "Cloud variables can only be set to a combination of numbers"
                )
            try:
                packet = {
                    "method": "set",
                    "name": (
                        "☁ " + variable if not variable.startswith("☁ ") else variable
                    ),
                    "value": str(value),
                    "user": self._client.username,
                    "project_id": str(self.project_id),
                }
                self._send_packet(packet)
                self.emit("outgoing", packet)
                self._timer = time.time()
                for cloud in self._cloudvariables:
                    if (
                            cloud.name == "☁ " + variable
                            if not variable.startswith("☁ ")
                            else variable
                    ):
                        cloud.value = value
                        self.emit("set", cloud)
                        break
            except (
                    BrokenPipeError,
                    websocket._exceptions.WebSocketConnectionClosedException,
            ):
                self.connect()
                time.sleep(0.1)
                self.set_cloud_variable(variable, value)
                return
        else:
            time.sleep(time.time() - self._timer)
            self.set_cloud_variable(variable, value)

    def _cloud_var_loop(self):
        while True:
            if self._ws.connected:
                response = self._ws.recv()
                response = json.loads(response)
                for cloud in self._cloudvariables:
                    if response["name"] == cloud.name:
                        cloud.value = response["value"]
                        self.emit("set", cloud)

            else:
                self.connect()

    def _start_cloud_var_loop(self):
        """Will start a new thread that looks for the cloud variables and appends their results onto cloudvariables"""
        thread = threading.Thread(target=self._cloud_var_loop)
        thread.start()

    def get_cloud_variable(self, name):
        try:
            var = next(
                x
                for x in self._cloudvariables
                if x.name == ("☁ " + name if not name.startswith("☁ ") else name)
            )
            return var.value
        except StopIteration:
            raise ValueError(
                "Variable '"
                + ("☁ " + name if not name.startswith("☁ ") else name)
                + "' is not in this project"
            )



import requests
import json

from .Users import AnotherUser, YourUser
from .Project import YourProject, AnotherProject


class Studio:
    def __init__(self, data, client):
        self.id = data["id"]
        self.title = data["title"]
        self.owner = data["owner"]
        self.description = data["description"]
        self.thumbnail_URL = data["image"]
        self.visible = data["visibility"] == "visibile"
        self.open_to_public = data["open_to_all"]

        self.created_timestamp = data["history"]["created"]
        self.last_modified_timestamp = data["history"]["modified"]

        self.follower_count = data["stats"]["followers"]

        self._client = client
        self._headers = {
            "x-csrftoken": self._client.csrf_token,
            "X-Token": self._client.token,
            "x-requested-with": "XMLHttpRequest",
            "Cookie": "scratchcsrftoken="
                      + self._client.csrf_token
                      + ";scratchlanguage=en;scratchsessionsid="
                      + self._client.session_id
                      + ";",
            "referer": "https://scratch.mit.edu/studios/" + str(self.id) + "/",
        }

    def add_project(self, project):
        project_id = project.id if isinstance(project, YourProject) or isinstance(project, AnotherProject) else project
        headers = self._headers
        headers["referer"] = "https://scratch.mit.edu/projects/" + str(project_id) + "/"
        requests.post(
            "https://api.scratch.mit.edu/studios/"
            + str(self.id)
            + "/project/"
            + project_id
            + "/",
            headers=headers,
        )

    def remove_project(self, project):
        project_id = project.id if isinstance(project, YourProject) or isinstance(project, AnotherProject) else project
        headers = self._headers
        headers["referer"] = "https://scratch.mit.edu/projects/" + str(project_id) + "/"
        requests.post(
            "https://api.scratch.mit.edu/studios/"
            + str(self.id)
            + "/project/"
            + project_id
            + "/",
            headers=headers,
        )

    def open_to_public(self):
        requests.put(
            "https://scratch.mit.edu/site-api/galleries/"
            + str(self.id)
            + "/mark/open/",
            headers=self._headers,
        )

    def close_to_public(self):
        requests.put(
            "https://scratch.mit.edu/site-api/galleries/"
            + str(self.id)
            + "/mark/closed/",
            headers=self._headers,
        )

    def follow(self):
        return requests.put(
            "https://scratch.mit.edu/site-api/users/bookmarkers/"
            + str(self.id)
            + "/add/?usernames="
            + self._client.username,
            headers=self._headers,
        ).json()

    def unfollow(self):
        return requests.put(
            "https://scratch.mit.edu/site-api/users/bookmarkers/"
            + str(self.id)
            + "/remove/?usernames="
            + self._client.username,
            headers=self._headers,
        ).json()

    def toggle_commenting(self):
        headers = self._headers
        headers["referer"] = (
                "https://scratch.mit.edu/studios/" + str(self.id) + "/comments/"
        )
        requests.post(
            "https://scratch.mit.edu/site-api/comments/gallery/"
            + str(self.id)
            + "/toggle-comments/",
            headers=headers,
        )

    def post_comment(self, content, parent_id="", commentee_id=""):
        headers = self._headers
        headers["referer"] = (
                "https://scratch.mit.edu/studios/" + str(self.id) + "/comments/"
        )
        data = {
            "commentee_id": commentee_id,
            "content": content,
            "parent_id": parent_id,
        }
        requests.post(
            "https://scratch.mit.edu/site-api/comments/gallery/"
            + str(self.id)
            + "/add/",
            headers=headers,
            data=json.dumps(data),
        )

    def delete_comment(self, comment_id, username):
        headers = self._headers
        headers["referer"] = (
                "https://scratch.mit.edu/studios/" + str(self.id) + "/comments/"
        )
        data = {"id": comment_id}
        requests.post(
            "https://scratch.mit.edu/site-api/comments/user/" + username + "/del/",
            headers=headers,
            data=json.dumps(data),
        )

    def report_comment(self, comment_id, username):
        headers = self._headers
        headers["referer"] = (
                "https://scratch.mit.edu/studios/" + str(self.id) + "/comments/"
        )
        data = {"id": comment_id}
        requests.post(
            "https://scratch.mit.edu/site-api/comments/user/" + username + "/rep/",
            headers=headers,
            data=json.dumps(data),
        )

    def invite_curator(self, user):
        username = user.username if isinstance(user, YourUser) or isinstance(user, AnotherUser) else user
        headers = self._headers
        headers["referer"] = (
                "https://scratch.mit.edu/studios/" + str(self.id) + "/curators/"
        )
        requests.put(
            "https://scratch.mit.edu/site-api/users/curators-in/"
            + str(self.id)
            + "/invite_curator/?usernames="
            + username,
            headers=headers,
        )

    def accept_curator(self):
        headers = self._headers
        headers["referer"] = (
                "https://scratch.mit.edu/studios/" + str(self.id) + "/curators/"
        )
        requests.put(
            "https://scratch.mit.edu/site-api/users/curators-in/"
            + str(self.id)
            + "/add/?usernames="
            + self._client.username,
            headers=headers,
        )

    def promote_curator(self, user):
        username = user.username if isinstance(user, YourUser) or isinstance(user, AnotherUser) else user
        headers = self._headers
        headers["referer"] = (
                "https://scratch.mit.edu/studios/" + str(self.id) + "/curators/"
        )
        requests.put(
            "https://scratch.mit.edu/site-api/users/curators-in/"
            + str(self.id)
            + "/promote/?usernames="
            + username,
            headers=headers,
        )

    def set_description(self, content):
        data = {"description": content}
        requests.put(
            "https://scratch.mit.edu/site-api/galleries/all/" + str(self.id) + "/",
            headers=self._headers,
            data=json.dumps(data),
        )
        self.description = content

    def set_title(self, content):
        data = {"title": content}
        requests.put(
            "https://scratch.mit.edu/site-api/galleries/all/" + str(self.id) + "/",
            headers=self._headers,
            data=json.dumps(data),
        )
        self.title = content



import re
import requests
import json
from .Exceptions import ScratchLoginException
from .Users import YourUser, AnotherUser
from .Project import YourProject, AnotherProject
from .Studios import Studio


class Session:
    def __init__(self, username=None, password=None):
        self.username = username
        self.password = password
        self._login()
        self.studio = Studio

    def _login(self, language="en"):
        if self.username is not None and self.password is not None:
            headers = {
                "x-csrftoken": "a",
                "x-requested-with": "XMLHttpRequest",
                "Cookie": f"scratchcsrftoken=a;scratchlanguage={language};",
                "referer": "https://scratch.mit.edu",
                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.101 Safari/537.36"
            }
            data = json.dumps({"username": self.username, "password": self.password})

            request = requests.post(
                "https://scratch.mit.edu/login/", data=data, headers=headers
            )

            try:
                self.session_id = re.search('"(.*)"', request.headers["Set-Cookie"]).group()
                self.token = request.json()[0]["token"]
            except AttributeError:
                raise ScratchLoginException("Your login password or username is incorrect")
            headers = {
                "x-requested-with": "XMLHttpRequest",
                "Cookie": f"scratchlanguage={language};permissions=%7B%7D;",
                "referer": "https://scratch.mit.edu",
            }
            request = requests.get("https://scratch.mit.edu/csrf_token/", headers=headers)
            self.csrf_token = re.search(
                "scratchcsrftoken=(.*?);", request.headers["Set-Cookie"]
            ).group(1)
            self.user = YourUser(self._get_user_json(self.username), self)
            self.auth = True
        else:
            self.session_id = "NoLogged"
            self.token = None
            self.csrf_token = "NoLogged"
            self.user = None
            self.auth = False

    def _get_user_json(self, username):
        return requests.get("https://api.scratch.mit.edu/users/" + username + "/").json()

    def get_user(self, username):
        i = self._get_user_json(username)
        print(i)
        return YourUser(i, self) if self.username == i["username"] else AnotherUser(i, self)

    def _get_project_json(self, id):
        return requests.get(
            "https://api.scratch.mit.edu/projects/" + str(id) + "/"
        ).json()

    def get_project(self, id):
        i = self._get_project_json(id)
        return YourProject(i, self) if self.username == i["username"] else AnotherProject(i, self)

    def _get_studio_json(self, id):
        return requests.get("https://api.scratch.mit.edu/studios/" + str(id) + "/").json()

    def get_studio(self, id):
        i = self._get_studio_json(id)
        return Studio(i, self)

    def _explore_projects_json(self, mode):
        return requests.get(
            "https://api.scratch.mit.edu/explore/projects/?mode="
            + mode
            + "&q=*"
        ).json()

    def explore_projects(self, mode="trending"):
        i = self._explore_projects_json(mode)
        return [
            YourProject(project, self) if project["author"]["username"] == self.username else AnotherProject(project,
                                                                                                             self)
            for project in i]

    def _search_projects_json(self, mode, query):
        return requests.get(
            "https://api.scratch.mit.edu/explore/projects/?mode="
            + mode
            + "&q="
            + query
        ).json()

    def search_projects(self, mode="popular", query='*'):
        i = self._search_projects_json(mode, query)
        return [
            YourProject(project, self) if project["author"]["username"] == self.username else AnotherProject(project,
                                                                                                             self)
            for project in i]



import requests
import json

from .Comments import ProjectComment


class AnotherProject:
    def __init__(self, data, client):
        self.id = data["id"]
        self.title = data["title"]
        self.description = data["description"]
        self.instructions = data["instructions"]
        self.visible = data["visibility"] == "visible"
        self.public = data["public"]
        self.comments_allowed = data["comments_allowed"]
        self.is_published = data["is_published"]
        self.author = (data["author"])
        self.thumbnail_URL = data["image"]

        self.created_timestamp = data["history"]["created"]
        self.last_modified_timestamp = data["history"]["modified"]
        self.shared_timestamp = data["history"]["shared"]

        self.view_count = data["stats"]["views"]
        self.love_count = data["stats"]["loves"]
        self.favorite_count = data["stats"]["favorites"]
        self.remix_count = data["stats"]["remixes"]

        self.parent = data["remix"].get("parent")
        self.root = data["remix"]["root"]
        self.is_remix = bool(self.parent)

        self._client = client
        self._headers = {
            "x-csrftoken": self._client.csrf_token,
            "X-Token": self._client.token,
            "x-requested-with": "XMLHttpRequest",
            "Cookie": "scratchcsrftoken="
                      + self._client.csrf_token
                      + ";scratchlanguage=en;scratchsessionsid="
                      + self._client.session_id
                      + ";",
            "referer": "https://scratch.mit.edu/projects/" + str(self.id) + "/",
        }
        self._json_headers = {
            "x-csrftoken": self._client.csrf_token,
            "X-Token": self._client.token,
            "x-requested-with": "XMLHttpRequest",
            "Cookie": "scratchcsrftoken="
                      + self._client.csrf_token
                      + ";scratchlanguage=en;scratchsessionsid="
                      + self._client.session_id
                      + ";",
            "referer": "https://scratch.mit.edu/projects/" + str(self.id) + "/",
            "accept": "application/json",
            "Content-Type": "application/json",
        }

    def get_comment(self, comment_id):
        data = requests.get(
            "https://api.scratch.mit.edu/users/"
            + self.author.username
            + "/projects/"
            + str(self.id)
            + "/comments/"
            + str(comment_id)
            + "/"
        ).json()
        return ProjectComment(self, data, self._client)

    def love(self):
        return requests.post(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/loves/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userLove"]

    def unlove(self):
        return requests.delete(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/loves/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userLove"]

    def favorite(self):
        return requests.post(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/favorites/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userFavorite"]

    def unfavorite(self):
        return requests.delete(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/favorites/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userFavorite"]

    def get_scripts(self):
        return requests.get(
            "https://projects.scratch.mit.edu/" + str(self.id) + "/"
        ).json()

    def get_remixes(self, all=False, limit=20, offset=0):
        if all:
            projects = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/projects/"
                    + str(self.id)
                    + "/remixes/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                projects += res
                if len(res) != 40:
                    break
                offset += 40
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]
        else:
            projects = requests.get(
                "https://api.scratch.mit.edu/projects/"
                + str(self.id)
                + "/remixes/"
                + "?limit="
                + str(limit)
                + "&offset="
                + str(offset)
            ).json()
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

    def get_studios(self, all=False, limit=20, offset=0):
        if all:
            studios = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/projects/"
                    + str(self.id)
                    + "/studios/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                studios += res
                if len(res) != 40:
                    break
                offset += 40
            return []
        else:
            return list(
                map(
                    self._client.studio,
                    requests.get(
                        "https://api.scratch.mit.edu/projects/"
                        + str(self.id)
                        + "/studios/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json(),
                )
            )

    def post_comment(self, content, parent_id="", commentee_id=""):
        data = {
            "commentee_id": commentee_id,
            "content": content,
            "parent_id": parent_id,
        }
        return requests.post(
            "https://api.scratch.mit.edu/proxy/comments/project/" + str(self.id) + "/",
            headers=self._json_headers,
            data=json.dumps(data),
        ).json()

    def get_comments(self, all=False, limit=20, offset=0):
        if all:
            comments = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.author.username
                    + "/projects/"
                    + str(self.id)
                    + "/comments/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                comments += res
                if len(res) != 40:
                    break
                offset += 40
            return [ProjectComment(self, i, self._client) for i in comments]
        else:
            comments = requests.get(
                        "https://api.scratch.mit.edu/users/"
                        + self.author.username
                        + "/projects/"
                        + str(self.id)
                        + "/comments/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json()
            return [ProjectComment(self, i, self._client) for i in comments]
    def report(self, category, reason, image=None):
        if not image:
            image = self.thumbnail_URL
        data = {"notes": reason, "report_category": category, "thumbnail": image}
        return requests.post(
            "https://api.scratch.mit.edu/proxy/comments/project/" + str(self.id) + "/",
            data=json.dumps(data),
            headers=self._json_headers,
        ).text

    def view(self):
        requests.post(
            "https://api.scratch.mit.edu/users/"
            + self.author.username
            + "/projects/"
            + str(self.id)
            + "/views/",
            headers=self._headers,
        )


class YourProject:
    def __init__(self, data, client):
        self.id = data["id"]
        self.title = data["title"]
        self.description = data["description"]
        self.instructions = data["instructions"]
        self.visible = data["visibility"] == "visible"
        self.public = data["public"]
        self.comments_allowed = data["comments_allowed"]
        self.is_published = data["is_published"]
        self.author = data["author"]
        self.thumbnail_URL = data["image"]

        self.created_timestamp = data["history"]["created"]
        self.last_modified_timestamp = data["history"]["modified"]
        self.shared_timestamp = data["history"]["shared"]

        self.view_count = data["stats"]["views"]
        self.love_count = data["stats"]["loves"]
        self.favorite_count = data["stats"]["favorites"]
        self.remix_count = data["stats"]["remixes"]

        self.parent = data["remix"].get("parent")
        self.root = data["remix"]["root"]
        self.is_remix = bool(self.parent)

        self._client = client
        self._headers = {
            "x-csrftoken": self._client.csrf_token,
            "X-Token": self._client.token,
            "x-requested-with": "XMLHttpRequest",
            "Cookie": "scratchcsrftoken="
                      + self._client.csrf_token
                      + ";scratchlanguage=en;scratchsessionsid="
                      + self._client.session_id
                      + ";",
            "referer": "https://scratch.mit.edu/projects/" + str(self.id) + "/",
        }
        self._json_headers = {
            "x-csrftoken": self._client.csrf_token,
            "X-Token": self._client.token,
            "x-requested-with": "XMLHttpRequest",
            "Cookie": "scratchcsrftoken="
                      + self._client.csrf_token
                      + ";scratchlanguage=en;scratchsessionsid="
                      + self._client.session_id
                      + ";",
            "referer": "https://scratch.mit.edu/projects/" + str(self.id) + "/",
            "accept": "application/json",
            "Content-Type": "application/json",
        }

    def get_comment(self, comment_id):
        data = requests.get(
            "https://api.scratch.mit.edu/users/"
            + self.author.username
            + "/projects/"
            + str(self.id)
            + "/comments/"
            + str(comment_id)
            + "/"
        ).json()
        return ProjectComment(self, data, self._client)

    def love(self):
        return requests.post(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/loves/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userLove"]

    def unlove(self):
        return requests.delete(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/loves/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userLove"]

    def favorite(self):
        return requests.post(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/favorites/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userFavorite"]

    def unfavorite(self):
        return requests.delete(
            "https://api.scratch.mit.edu/proxy/projects/"
            + str(self.id)
            + "/favorites/user/"
            + self._client.username,
            headers=self._headers,
        ).json()["userFavorite"]

    def get_scripts(self):
        return requests.get(
            "https://projects.scratch.mit.edu/" + str(self.id) + "/"
        ).json()

    def get_remixes(self, all=False, limit=20, offset=0):
        if all:
            projects = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/projects/"
                    + str(self.id)
                    + "/remixes/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                projects += res
                if len(res) != 40:
                    break
                offset += 40
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]
        else:
            projects = requests.get(
                "https://api.scratch.mit.edu/projects/"
                + str(self.id)
                + "/remixes/"
                + "?limit="
                + str(limit)
                + "&offset="
                + str(offset)
            ).json()
            return [
                YourProject(i, self._client) if i["author"]["username"] == self._client.username else AnotherProject(i,
                                                                                                                     self._client)
                for i in projects]

    def get_studios(self, all=False, limit=20, offset=0):
        if all:
            studios = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/projects/"
                    + str(self.id)
                    + "/studios/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                studios += res
                if len(res) != 40:
                    break
                offset += 40
            return list(map(self._client.studio, studios))
        else:
            return list(
                map(
                    self._client.studio,
                    requests.get(
                        "https://api.scratch.mit.edu/projects/"
                        + str(self.id)
                        + "/studios/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json(),
                )
            )

    def post_comment(self, content, parent_id="", commentee_id=""):
        data = {
            "commentee_id": commentee_id,
            "content": content,
            "parent_id": parent_id,
        }
        return requests.post(
            "https://api.scratch.mit.edu/proxy/comments/project/" + str(self.id) + "/",
            headers=self._json_headers,
            data=json.dumps(data),
        ).json()

    def get_comments(self, all=False, limit=20, offset=0):
        if all:
            comments = []
            offset = 0
            while True:
                res = requests.get(
                    "https://api.scratch.mit.edu/users/"
                    + self.author.username
                    + "/projects/"
                    + str(self.id)
                    + "/comments/"
                    + "?limit=40&offset="
                    + str(offset)
                ).json()
                comments += res
                if len(res) != 40:
                    break
                offset += 40
            return [ProjectComment(self, i, self._client) for i in comments]
        else:
            comments = requests.get(
                        "https://api.scratch.mit.edu/users/"
                        + self.author.username
                        + "/projects/"
                        + str(self.id)
                        + "/comments/"
                        + "?limit="
                        + str(limit)
                        + "&offset="
                        + str(offset)
                    ).json()
            return [ProjectComment(self, i, self._client) for i in comments]

    def toggle_commenting(self):
        data = {"comments_allowed": not self.comments_allowed}
        self.comments_allowed = not self.comments_allowed
        return self._client._to_project(
            requests.put(
                "https://api.scratch.mit.edu/projects/" + str(self.id) + "/",
                data=json.dumps(data),
                headers=self._json_headers,
            ).json()
        )

    def turn_on_commenting(self):
        data = {"comments_allowed": True}
        self.comments_allowed = True
        return self._client._to_project(
            requests.put(
                "https://api.scratch.mit.edu/projects/" + str(self.id) + "/",
                data=json.dumps(data),
                headers=self._json_headers,
            ).json()
        )

    def turn_off_commenting(self):
        data = {"comments_allowed": False}
        self.comments_allowed = False
        return self._client._to_project(
            requests.put(
                "https://api.scratch.mit.edu/projects/" + str(self.id) + "/",
                data=json.dumps(data),
                headers=self._json_headers,
            ).json()
        )

    def report(self, category, reason, image=None):
        if not image:
            image = self.thumbnail_URL
        data = {"notes": reason, "report_category": category, "thumbnail": image}
        return requests.post(
            "https://api.scratch.mit.edu/proxy/comments/project/" + str(self.id) + "/",
            data=json.dumps(data),
            headers=self._json_headers,
        ).text

    def unshare(self):

        requests.put(
            "https://api.scratch.mit.edu/proxy/projects/" + str(self.id) + "/unshare/",
            headers=self._json_headers,
        )

    def share(self):
        requests.put(
            "https://api.scratch.mit.edu/proxy/projects/" + str(self.id) + "/share/",
            headers=self._json_headers,
        )

    def view(self):
        requests.post(
            "https://api.scratch.mit.edu/users/"
            + self.author.username
            + "/projects/"
            + str(self.id)
            + "/views/",
            headers=self._headers,
        )

    def set_thumbnail(self, file):
        image = open(file, "rb")
        requests.post(
            "https://scratch.mit.edu/internalapi/project/thumbnail/"
            + str(self.id)
            + "/set/",
            data=image.read(),
            headers=self._headers,
        )

    def set_title(self, title):
        data = {"title": title}
        requests.put(
            "https://api.scratch.mit.edu/projects/"
            + str(self.id),
            data=json.dumps(data),
            headers=self._json_headers,
        )



import requests
import json


class YourUserProfile:
    def __init__(self, data, user):
        self.user = user
        self.username = user.username
        self.id = data["id"]
        self.avatar_URL = data["images"]["90x90"]
        self.bio = data["bio"]
        self.status = data["status"]
        self.country = data["country"]

    def set_bio(self, content):
        data = {"bio": content}
        return requests.put(
            "https://scratch.mit.edu/site-api/users/all/" + self.username + "/",
            data=json.dumps(data),
            headers=self.user._headers,
        )

    def set_status(self, content):
        data = {"status": content}
        return requests.put(
            "https://scratch.mit.edu/site-api/users/all/" + self.username + "/",
            data=json.dumps(data),
            headers=self.user._headers,
        )

    def set_featured_project(self, label, project):
        label = (
            {
                "featured_project": "",
                "featured_tutorial": 0,
                "work_in_progress": 1,
                "remix_this": 2,
                "my_favorite_things": 3,
                "why_i_scratch": 4,
            }
        )[label]
        project_id = project
        data = {"featured_project": project_id, "featured_project_label": label}
        requests.put(
            "https://scratch.mit.edu/site-api/users/all/" + self.username + "/",
            data=json.dumps(data),
            headers=self.user._headers,
        )


class AnotherUserProfile:
    def __init__(self, data, user):
        self.user = user
        self.username = user.username
        self.id = data["id"]
        self.avatar_URL = data["images"]["90x90"]
        self.bio = data["bio"]
        self.status = data["status"]
        self.country = data["country"]



class ScratchException(Exception):
    pass


class ScratchAuthException(ScratchException):
    pass


class ScratchNoAuthException(ScratchAuthException):
    pass


class ScratchLoginException(ScratchAuthException):
    pass



from setuptools import setup
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
setup(name='scratchplus',
      version='0.99',
      description='API wrapper for scratch',
      packages=['scratchplus'],
      author_email='thefinalspacestudio@gmail.com',
      author="gagarinten",
      zip_safe=False,
      url='https://github.com/noverd/ScratchPlus',
      install_requires=["requests",
                        "websocket", "pymitter"],
      long_description=long_description,
      long_description_content_type='text/markdown'
      )